@@ -1,9 +1,10 @@
 use anyhow::Result;
-use async_std::task::sleep;
+use async_std::task::{sleep, spawn};
 use igd::aio::search_gateway;
 use igd::AddPortError::{self, PortInUse};
 use std::net::TcpStream;
 use std::net::{IpAddr, Ipv4Addr, SocketAddrV4};
+use std::sync::mpsc::{channel, Receiver};
 use std::time::Duration;
 
 #[derive(thiserror::Error, Debug)]
@@ -12,44 +13,48 @@ pub enum Error {
   NotIpv4,
 }
 
-pub async fn daemon(
-  name: &str,
+pub fn daemon<'a>(
+  name: String,
   port: u16,
   duration: u32,
-  resolve: Box<dyn Fn(SocketAddrV4, u16, Ipv4Addr) + Send>,
-  reject: Box<dyn Fn(&dyn std::error::Error) + Send>,
-) {
+) -> Receiver<Result<(u16, Ipv4Addr, SocketAddrV4), String>> {
   let mut local_ip = Ipv4Addr::UNSPECIFIED;
   let mut pre_gateway = SocketAddrV4::new(local_ip, 0);
 
   let seconds = Duration::from_secs(duration.into());
   let duration = duration + 9;
   let mut ext_port = port;
-  loop {
-    match upnp(name, port, ext_port, duration).await {
-      Ok((gateway, ip)) => {
-        if ip != local_ip || gateway != pre_gateway {
-          local_ip = ip;
-          pre_gateway = gateway;
-          resolve(gateway, ext_port, ip);
+  let (s, r) = channel();
+
+  spawn(async move {
+    loop {
+      match upnp(&name, port, ext_port, duration).await {
+        Ok((gateway, ip)) => {
+          if ip != local_ip || gateway != pre_gateway {
+            local_ip = ip;
+            pre_gateway = gateway;
+            let _ = s.send(Ok((ext_port, ip, gateway)));
+          }
         }
-      }
-      Err(err) => {
-        let err = err.root_cause();
-        if let Some(PortInUse) = err.downcast_ref::<AddPortError>() {
-          if ext_port == 65535 {
-            ext_port = 1025;
+        Err(err) => {
+          let err = err.root_cause();
+          if let Some(PortInUse) = err.downcast_ref::<AddPortError>() {
+            if ext_port == 65535 {
+              ext_port = 1025;
+            } else {
+              ext_port += 1;
+            }
+            continue;
           } else {
-            ext_port += 1;
+            local_ip = Ipv4Addr::UNSPECIFIED;
+            let _ = s.send(Err(err.to_string()));
           }
-          continue;
-        } else {
-          reject(err);
         }
       }
+      sleep(seconds).await;
     }
-    sleep(seconds).await;
-  }
+  });
+  r
 }
 
 pub async fn upnp(
